package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class ModuleOpens implements Cloneable, Node
{
    private final int opensIndex;
    private final int opensFlags;
    private final int opensToCount;
    private final int[] opensToIndex;
    
    ModuleOpens(final DataInput file) throws IOException {
        this.opensIndex = file.readUnsignedShort();
        this.opensFlags = file.readUnsignedShort();
        this.opensToCount = file.readUnsignedShort();
        this.opensToIndex = new int[this.opensToCount];
        for (int i = 0; i < this.opensToCount; ++i) {
            this.opensToIndex[i] = file.readUnsignedShort();
        }
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitModuleOpens(this);
    }
    
    public ModuleOpens copy() {
        try {
            return (ModuleOpens)this.clone();
        }
        catch (final CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public void dump(final DataOutputStream file) throws IOException {
        file.writeShort(this.opensIndex);
        file.writeShort(this.opensFlags);
        file.writeShort(this.opensToCount);
        for (final int entry : this.opensToIndex) {
            file.writeShort(entry);
        }
    }
    
    @Override
    public String toString() {
        return "opens(" + this.opensIndex + ", " + this.opensFlags + ", " + this.opensToCount + ", ...)";
    }
    
    public String toString(final ConstantPool constantPool) {
        final StringBuilder buf = new StringBuilder();
        final String packageName = constantPool.constantToString(this.opensIndex, (byte)20);
        buf.append(Utility.compactClassName(packageName, false));
        buf.append(", ").append(String.format("%04x", this.opensFlags));
        buf.append(", to(").append(this.opensToCount).append("):\n");
        for (final int index : this.opensToIndex) {
            final String moduleName = constantPool.getConstantString(index, (byte)19);
            buf.append("      ").append(Utility.compactClassName(moduleName, false)).append("\n");
        }
        return buf.substring();
    }
}
